/***************************************************************************
                          fractaldisplay.h  -  description
                             -------------------
    begin                : Sam Mai 10 2003
    copyright            : (C) 2003 by Thomas Dreibholz
    email                : dreibh@exp-math.uni-essen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FRACTALDISPLAY_H
#define FRACTALDISPLAY_H

#include <qwidget.h>
#include <qimage.h>
#include <qpainter.h>
#include <qdatetime.h>

#include <kurl.h>

/**
  *@author Thomas Dreibholz
  */

class FractalDisplay : public QWidget {
   Q_OBJECT
   public: 
   FractalDisplay(QWidget *parent=0, const char *name=0);
   ~FractalDisplay();

   bool reset(const unsigned int width, const unsigned int height);
   inline void setPoint(const unsigned int x, const unsigned int y, const QRgb color) {
      Image->setPixel(x, y, color);
   }
   inline QImage* image() {
      return(Image);
   }
   inline int imageWidth() {
      return(Image->width());
   }
   inline int imageHeight() {
      return(Image->height());
   }
   inline unsigned int offsetX() {
      return(OffsetX);
   }
   inline unsigned int offsetY() {
      return(OffsetY);
   }
   inline void setOffsetX(const unsigned int offsetX) {
      OffsetX = offsetX;
   }
   inline void setOffsetY(const unsigned int offsetY) {
      OffsetY = offsetY;
   }
   inline bool savePicture(QString &path, const char *format) {
       return Image->save(path, format); }

   signals:
   void offsetUpdate(int newOffsetX, int newOffsetY);
   void selection(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2);
   void zoom();

   protected:
   void resizeEvent(QResizeEvent* resizeEvent);
   void paintEvent(QPaintEvent* paintEvent);
   void mousePressEvent(QMouseEvent* mouseEvent);
   void mouseReleaseEvent(QMouseEvent* mouseEvent);
   void mouseMoveEvent(QMouseEvent* mouseEvent);

   private:
   void getMarkPosition(QMouseEvent* mouseEvent, int& x, int& y);
   void drawMarkerRect(QPainter* painter, int x1, int y1, int x2, int y2, bool draw = true);
   void showMarker();

   QImage*      Image;
   unsigned int OffsetX;
   unsigned int OffsetY;
   QTime        LastOffsetUpdate;
   int          MarkX1;
   int          MarkY1;
   int          MarkX2;
   int          MarkY2;
   int          OldMarkX1;
   int          OldMarkY1;
   int          OldMarkX2;
   int          OldMarkY2;
   bool         Marking;
};

#endif
